@@ -1,7 +1,7 @@
 """This module defines utilities for transforming
 complete Forms into new related Forms."""
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -21,9 +21,9 @@ complete Forms into new related Forms."""
 # Modified by Anders Logg, 2008-2009.
 # Modified by Garth N. Wells, 2010.
 # Modified by Marie E. Rognes, 2010.
-#
-# First added:  2008-10-01
-# Last changed: 2012-04-12
+
+from six import iteritems
+from six.moves import xrange as range
 
 from ufl.common import product
 from ufl.log import error, warning, debug
@@ -36,13 +36,27 @@ from ufl.constantvalue import Zero
 from ufl.algebra import Sum
 
 # Other algorithms:
-from ufl.algorithms.traversal import traverse_terminals
-from ufl.algorithms.analysis import extract_arguments
-from ufl.algorithms.transformer import Transformer, transform_integrands
+from ufl.algorithms.map_integrands import map_integrands
+from ufl.algorithms.transformer import Transformer
 from ufl.algorithms.replace import replace
 
-def zero(e):
-    return Zero(e.shape(), e.free_indices(), e.index_dimensions())
+
+# FIXME: Don't use this below, it makes partextracter more expensive than necessary
+def _expr_has_terminal_types(expr, ufl_types):
+    input = [expr]
+    while input:
+        e = input.pop()
+        ops = e.ufl_operands
+        if ops:
+            input.extend(ops)
+        elif isinstance(e, ufl_types):
+            return True
+    return False
+
+
+def zero_expr(e):
+    return Zero(e.ufl_shape, e.ufl_free_indices, e.ufl_index_dimensions)
+
 
 class PartExtracter(Transformer):
     """
@@ -57,9 +71,7 @@ class PartExtracter(Transformer):
     def expr(self, x):
         """The default is a nonlinear operator not accepting any
         Arguments among its children."""
-
-        # FIXME: This check makes this an O(n^2) algorithm...
-        if any(isinstance(t, Argument) for t in traverse_terminals(x)):
+        if _expr_has_terminal_types(x, Argument):
             error("Found Argument in %s, this is an invalid expression." % repr(x))
         return (x, set())
 
@@ -71,13 +83,13 @@ class PartExtracter(Transformer):
         "Return relevant parts of this variable."
 
         # Extract parts/provides from this variable's expression
-        expression, label = x.operands()
+        expression, label = x.ufl_operands
         part, provides = self.visit(expression)
 
         # If the extracted part is zero or we provide more than we
         # want, return zero
         if isinstance(part, Zero) or (provides - self._want):
-            return (zero(x), set())
+            return (zero_expr(x), set())
 
         # Reuse varible if possible (or reconstruct from part)
         x = self.reuse_if_possible(x, part, label)
@@ -88,11 +100,11 @@ class PartExtracter(Transformer):
         "Return itself unless itself provides too much."
 
         # An argument provides itself
-        provides = set((x,))
+        provides = {x}
 
         # If we provide more than we want, return zero
         if provides - self._want:
-            return (zero(x), set())
+            return (zero_expr(x), set())
 
         return (x, provides)
 
@@ -127,7 +139,8 @@ class PartExtracter(Transformer):
         parts_that_provide = {}
 
         # 1. Skip terms that provide too much
-        original_terms = x.operands()
+        original_terms = x.ufl_operands
+        assert len(original_terms) == 2
         for term in original_terms:
 
             # Visit this term in the sum
@@ -147,11 +160,11 @@ class PartExtracter(Transformer):
 
         # 2. If there are no remaining terms, return zero
         if not parts_that_provide:
-            return (zero(x), set())
+            return (zero_expr(x), set())
 
         # 3. Return the terms that provide the biggest set
         most_provided = frozenset()
-        for (provideds, parts) in parts_that_provide.iteritems(): # TODO: Just sort instead?
+        for (provideds, parts) in iteritems(parts_that_provide): # TODO: Just sort instead?
 
             # Throw error if size of sets are equal (and not zero)
             if len(provideds) == len(most_provided) and len(most_provided):
@@ -161,10 +174,10 @@ class PartExtracter(Transformer):
                 most_provided = provideds
 
         terms = parts_that_provide[most_provided]
-        if len(terms) == len(original_terms):
+        if len(terms) == 2:
             x = self.reuse_if_possible(x, *terms)
         else:
-            x = Sum(*terms)
+            x, = terms
 
         return (x, most_provided)
 
@@ -179,7 +192,7 @@ class PartExtracter(Transformer):
 
             # If any factor is zero, return
             if isinstance(factor, Zero):
-                return (zero(x), set())
+                return (zero_expr(x), set())
 
             # Add factor to factors and extend provides
             factors.append(factor)
@@ -187,7 +200,7 @@ class PartExtracter(Transformer):
 
             # If we provide more than we want, return zero
             if provides - self._want:
-                return (zero(x), provides)
+                return (zero_expr(x), provides)
 
         # Reuse product if possible (or reconstruct from factors)
         x = self.reuse_if_possible(x, *factors)
@@ -203,10 +216,10 @@ class PartExtracter(Transformer):
         "Return parts_of_numerator/denominator."
 
         # Get numerator and denominator
-        numerator, denominator = x.operands()
+        numerator, denominator = x.ufl_operands
 
         # Check for Arguments in the denominator
-        if any(isinstance(t, Argument) for t in traverse_terminals(denominator)):
+        if _expr_has_terminal_types(denominator, Argument):
             error("Found Argument in denominator of %s , this is an invalid expression." % repr(x))
 
         # Visit numerator
@@ -215,7 +228,7 @@ class PartExtracter(Transformer):
         # If numerator is zero, return zero. (No need to check whether
         # it provides too much, already checked by visit.)
         if isinstance(numerator_parts, Zero):
-            return (zero(x), set())
+            return (zero_expr(x), set())
 
         # Reuse x if possible, otherwise reconstruct from (parts of)
         # numerator and denominator
@@ -234,7 +247,7 @@ class PartExtracter(Transformer):
         # Discard if part is zero. (No need to check whether we
         # provide too much, already checked by children.)
         if isinstance(part, Zero):
-            return (zero(x), set())
+            return (zero_expr(x), set())
 
         x = self.reuse_if_possible(x, part)
 
@@ -255,13 +268,13 @@ class PartExtracter(Transformer):
         """Return parts of expression belonging to this indexed
         expression."""
 
-        expression, index = x.operands()
+        expression, index = x.ufl_operands
         part, provides = self.visit(expression)
 
         # Return zero if extracted part is zero. (The expression
         # should already have checked if it provides too much.)
         if isinstance(part, Zero):
-            return (zero(x), set())
+            return (zero_expr(x), set())
 
         # Reuse x if possible (or reconstruct by indexing part)
         x = self.reuse_if_possible(x, part, index)
@@ -272,7 +285,6 @@ class PartExtracter(Transformer):
     indexed = linear_indexed_type
     index_sum = linear_indexed_type
     component_tensor = linear_indexed_type
-    spatial_derivative = linear_indexed_type
 
     def list_tensor(self, x, *ops):
         # list_tensor is a visit-children-first handler. ops contains
@@ -304,7 +316,11 @@ def compute_form_with_arity(form, arity, arguments=None):
 
     # Extract all arguments in form
     if arguments is None:
-        arguments = extract_arguments(form)
+        arguments = form.arguments()
+
+    parts = [arg.part() for arg in arguments]
+    if set(parts) - {None}:
+        error("compute_form_with_arity cannot handle parts.")
 
     if len(arguments) < arity:
         warning("Form has no parts with arity %d." % arity)
@@ -321,15 +337,18 @@ def compute_form_with_arity(form, arity, arguments=None):
         if provides == sub_arguments:
             return e
         return Zero()
-    res = transform_integrands(form, _transform)
-    return res
+    return map_integrands(_transform, form)
 
 def compute_form_arities(form):
     """Return set of arities of terms present in form."""
     #ufl_assert(form.is_preprocessed(), "Assuming a preprocessed form.")
 
     # Extract all arguments present in form
-    arguments = extract_arguments(form)
+    arguments = form.arguments()
+
+    parts = [arg.part() for arg in arguments]
+    if set(parts) - {None}:
+        error("compute_form_arities cannot handle parts.")
 
     arities = set()
     for arity in range(len(arguments)+1):
@@ -382,7 +401,11 @@ def compute_form_action(form, coefficient):
     # TODO: Check whatever makes sense for coefficient
 
     # Extract all arguments
-    arguments = extract_arguments(form)
+    arguments = form.arguments()
+
+    parts = [arg.part() for arg in arguments]
+    if set(parts) - {None}:
+        error("compute_form_action cannot handle parts.")
 
     # Pick last argument (will be replaced)
     u = arguments[-1]
@@ -405,7 +428,12 @@ def compute_energy_norm(form, coefficient):
     Arguments, and one additional Coefficient at the
     end if no coefficient has been provided.
     """
-    arguments = extract_arguments(form)
+    arguments = form.arguments()
+
+    parts = [arg.part() for arg in arguments]
+    if set(parts) - {None}:
+        error("compute_energy_norm cannot handle parts.")
+
     ufl_assert(len(arguments) == 2, "Expecting bilinear form.")
     v, u = arguments
     e = u.element()
@@ -422,19 +450,34 @@ def compute_energy_norm(form, coefficient):
 def compute_form_adjoint(form, reordered_arguments=None):
     """Compute the adjoint of a bilinear form.
 
-    This works simply by changing the ordering (count) of the two arguments.
+    This works simply by swapping the number and part of the two arguments,
+    but keeping their elements and places in the integrand expressions.
     """
-    arguments = extract_arguments(form)
+    arguments = form.arguments()
+
+    parts = [arg.part() for arg in arguments]
+    if set(parts) - {None}:
+        error("compute_form_adjoint cannot handle parts.")
+
     ufl_assert(len(arguments) == 2, "Expecting bilinear form.")
 
     v, u = arguments
-    ufl_assert(v.count() < u.count(), "Mistaken assumption in code!")
+    ufl_assert(v.number() < u.number(), "Mistaken assumption in code!")
 
     if reordered_arguments is None:
-        reordered_arguments = (Argument(u.element()), Argument(v.element()))
-    reordered_u, reordered_v = reordered_arguments
-    ufl_assert(reordered_u.count() < reordered_v.count(),
+        reordered_u = u.reconstruct(number=v.number(), part=v.part())
+        reordered_v = v.reconstruct(number=u.number(), part=u.part())
+    else:
+        reordered_u, reordered_v = reordered_arguments
+
+    ufl_assert(reordered_u.number() < reordered_v.number(),
                "Ordering of new arguments is the same as the old arguments!")
+
+    ufl_assert(reordered_u.part() == v.part(),
+               "Ordering of new arguments is the same as the old arguments!")
+    ufl_assert(reordered_v.part() == u.part(),
+               "Ordering of new arguments is the same as the old arguments!")
+
     ufl_assert(reordered_u.element() == u.element(),
                "Element mismatch between new and old arguments (trial functions).")
     ufl_assert(reordered_v.element() == v.element(),
