@@ -32,7 +32,7 @@ from ufl.permutation import compute_indices
 
 # All classes:
 from ufl.variable import Variable
-from ufl.indexing import Index, FixedIndex
+from ufl.core.multiindex import Index, FixedIndex
 from ufl.indexed import Indexed
 from ufl.tensors import ListTensor, ComponentTensor
 from ufl.algebra import Sum, Product, Division, Power, Abs
@@ -48,8 +48,7 @@ from ufl.classes import terminal_classes
 from ufl.geometry import Domain
 
 # Other algorithms:
-from ufl.algorithms.preprocess import preprocess
-from ufl.algorithms.analysis import extract_variables
+from ufl.algorithms.compute_form_data import compute_form_data
 from ufl.algorithms.formfiles import load_forms
 from ufl.algorithms.latextools import align, document, verbatim
 
@@ -60,6 +59,24 @@ from ufl.algorithms.graph import build_graph, partition, extract_outgoing_vertex
 # TODO: Maybe this can be cleaner written using the graph utilities
 
 
+
+def _extract_variables(a):
+    """Build a list of all Variable objects in a,
+    which can be a Form, Integral or Expr.
+    The ordering in the list obeys dependency order."""
+    handled = set()
+    variables = []
+    for e in iter_expressions(a):
+        for o in post_traversal(e):
+            if isinstance(o, Variable):
+                expr, label = o.ufl_operands
+                if not label in handled:
+                    variables.append(o)
+                    handled.add(label)
+    return variables
+
+
+
 # --- Tools for LaTeX rendering of UFL expressions ---
 
 # TODO: Finish precedence mapping
@@ -112,7 +129,7 @@ def format_multi_index(ii, formatstring="%s"):
 
 def bfname(i, p):
     s = "" if p is None else (",%d"%(p,))
-    return "{v_h^{%d%s}}" % (i,s)
+    return "{v_h^{%d%s}}" % (i, s)
 
 def cfname(i):
     return "{w_h^%d}" % i
@@ -128,12 +145,12 @@ class Expression2LatexHandler(Transformer):
     # --- Terminal objects ---
 
     def scalar_value(self, o):
-        if o.shape():
+        if o.ufl_shape:
             return r"{\mathbf %s}" % o._value
         return "{%s}" % o._value
 
     def zero(self, o):
-        return "0" if not o.shape() else r"{\mathbf 0}"
+        return "0" if not o.ufl_shape else r"{\mathbf 0}"
 
     def identity(self, o):
         return r"{\mathbf I}"
@@ -162,7 +179,7 @@ class Expression2LatexHandler(Transformer):
 
     def variable(self, o):
         # TODO: Ensure variable has been handled
-        e, l = o.operands()
+        e, l = o.ufl_operands
         return "s_{%d}" % l._count
 
     # --- Non-terminal objects ---
@@ -304,14 +321,14 @@ class Expression2LatexHandler(Transformer):
         return "sym{%s}" % par(A) # TODO: Get built-in function syntax like \sin for this
 
     def list_tensor(self, o):
-        shape = o.shape()
+        shape = o.ufl_shape
         if len(shape) == 1:
-            ops = [self.visit(op) for op in o.operands()]
+            ops = [self.visit(op) for op in o.ufl_operands]
             l = " \\\\ \n ".join(ops)
         elif len(shape) == 2:
             rows = []
-            for row in o.operands():
-                cols = (self.visit(op) for op in row.operands())
+            for row in o.ufl_operands:
+                cols = (self.visit(op) for op in row.ufl_operands)
                 rows.append( " & \n ".join(cols) )
             l = " \\\\ \n ".join(rows)
         else:
@@ -365,6 +382,12 @@ class Expression2LatexHandler(Transformer):
         l += "\\end{cases}"
         return l
 
+    def min_value(self, o, a, b):
+        return "min(%s, %s)" % (a, b)
+
+    def max_value(self, o, a, b):
+        return "max(%s, %s)" % (a, b)
+
     def expr(self, o):
         error("Missing handler for type %s" % str(type(o)))
 
@@ -426,7 +449,7 @@ def form2latex(form, formdata):
     for f in formdata.original_arguments:
         i = f.number()
         p = f.part()
-        lines.append("%s = %s \\in V_h^{%d} " % (argument_names[(i,p)], bfname(i,p), i)) # FIXME: Handle part in V_h
+        lines.append("%s = %s \\in V_h^{%d} " % (argument_names[(i, p)], bfname(i, p), i)) # FIXME: Handle part in V_h
     for i, f in enumerate(formdata.original_coefficients):
         lines.append("%s = %s \\in W_h^{%d} " % (coefficient_names[i], cfname(i), i))
     if lines:
@@ -451,7 +474,7 @@ def form2latex(form, formdata):
                "Not handling non-standard integral types!")
     lines = []
     for itg in integrals:
-        variables = extract_variables(itg.integrand())
+        variables = _extract_variables(itg.integrand())
         for v in variables:
             l = v._label
             if not l in handled_variables:
@@ -506,7 +529,7 @@ def form2latex(form, formdata):
 def ufl2latex(expression):
     "Generate LaTeX code for a UFL expression or form (wrapper for form2latex and expression2latex)."
     if isinstance(expression, Form):
-        form_data = expression.compute_form_data()
+        form_data = compute_form_data(expression)
         preprocessed_form = form_data.preprocessed_form
         return form2latex(preprocessed_form, form_data)
     else:
@@ -546,7 +569,7 @@ def dependency_sorting(deplist, rank):
     # Permutations of 0/1 dependence of arguments
     indices = compute_indices((2,)*rank)
     for bfs in indices[1:]: # skip (0,...,0), already handled that
-        for i, bf in reversed(list(enumerate(bfs))):
+        for i, bf in reversed(enumerate(bfs)):
             n = "v%d" % i
             if bf:
                 if n in state:
@@ -570,7 +593,7 @@ def dependency_sorting(deplist, rank):
 
     indices = compute_indices((2,)*rank)
     for bfs in indices[1:]: # skip (0,...,0), already handled that
-        for i, bf in reversed(list(enumerate(bfs))):
+        for i, bf in reversed(enumerate(bfs)):
             n = "v%d" % i
             if bf:
                 state.add(n)
@@ -601,7 +624,7 @@ def code2latex(G, partitions, formdata):
     Vout = extract_outgoing_vertex_connections(G)
 
     # Sort dependency sets in a sensible way (preclude to a good quadrature code generator)
-    deplistlist = dependency_sorting(partitions.keys(), len(bfn))
+    deplistlist = dependency_sorting(list(partitions.keys()), len(bfn))
 
     def format_v(i):
         return "s_{%d}" % i
@@ -617,7 +640,7 @@ def code2latex(G, partitions, formdata):
                 vl = format_v(iv)
                 args = ", ".join(format_v(i) for i in vout)
                 if args:
-                    el = r"{\mbox{%s}}(%s)" % (v._uflclass.__name__, args)
+                    el = r"{\mbox{%s}}(%s)" % (v._ufl_class_.__name__, args)
                 else: # terminal
                     el = r"{\mbox{%s}}" % (repr(v),)
                 lines.append((vl, "= " + el))
@@ -668,7 +691,7 @@ def forms2latexdocument(forms, uflfilename, compile=False):
     for form in forms:
 
         # Compute form data
-        form_data = preprocess(form)
+        form_data = compute_form_data(form)
 
         # Generate LaTex code
         title = "Form %s" % form_data.name
