@@ -1,6 +1,6 @@
 "Various high level ways to transform a complete Form into a new Form."
 
-# Copyright (C) 2008-2013 Martin Sandve Alnes
+# Copyright (C) 2008-2014 Martin Sandve Alnes
 #
 # This file is part of UFL.
 #
@@ -18,26 +18,27 @@
 # along with UFL. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Anders Logg, 2009
-#
-# First added:  2008-03-14
-# Last changed: 2013-01-02
 
-from itertools import izip
+from six import iteritems
+from six.moves import zip
+from six.moves import xrange as range
+
 from ufl.log import error
 from ufl.assertions import ufl_assert
-from ufl.form import Form
-from ufl.expr import Expr
+from ufl.form import Form, as_form
+from ufl.core.expr import Expr
 from ufl.split_functions import split
-from ufl.operatorbase import Tuple
+from ufl.exprcontainers import ExprList, ExprMapping
 from ufl.variable import Variable
 from ufl.finiteelement import MixedElement
 from ufl.argument import Argument
 from ufl.coefficient import Coefficient
 from ufl.differentiation import CoefficientDerivative
-from ufl.constantvalue import is_true_ufl_scalar
+from ufl.constantvalue import is_true_ufl_scalar, as_ufl
 from ufl.indexed import Indexed
-from ufl.indexing import FixedIndex, MultiIndex
+from ufl.core.multiindex import FixedIndex, MultiIndex
 from ufl.tensors import as_tensor
+from ufl.sorting import sorted_expr
 
 # An exception to the rule that ufl.* does not depend on ufl.algorithms.* ...
 from ufl.algorithms import compute_form_adjoint, \
@@ -47,7 +48,7 @@ from ufl.algorithms import compute_form_adjoint, \
                            compute_form_rhs, \
                            compute_form_functional, \
                            expand_derivatives, \
-                           as_form
+                           extract_arguments
 
 # Part of the external interface
 from ufl.algorithms import replace
@@ -102,10 +103,8 @@ def action(form, coefficient=None):
 
 def energy_norm(form, coefficient=None):
     """UFL form operator:
-    Given a bilinear form, return a linear form
-    with an additional coefficient, representing the
-    action of the form on the coefficient. This can be
-    used for matrix-free methods."""
+    Given a bilinear form a and a coefficient f,
+    return the functional a(f,f)."""
     form = as_form(form)
     form = expand_derivatives(form)
     return compute_energy_norm(form, coefficient)
@@ -139,9 +138,9 @@ def set_list_item(li, i, v):
     # Set item in innermost list
     li[i[-1]] = v
 
-def _handle_derivative_arguments(coefficient, argument):
+def _handle_derivative_arguments(form, coefficient, argument):
     # Wrap single coefficient in tuple for uniform treatment below
-    if isinstance(coefficient, (list,tuple)):
+    if isinstance(coefficient, (list, tuple)):
         coefficients = tuple(coefficient)
     else:
         coefficients = (coefficient,)
@@ -151,36 +150,52 @@ def _handle_derivative_arguments(coefficient, argument):
         if not all(isinstance(c, Coefficient) for c in coefficients):
             error("Can only create arguments automatically for non-indexed coefficients.")
 
+        # Get existing arguments from form and position the new one with the next argument number
+        if isinstance(form, Form):
+            form_arguments = form.arguments()
+        else:
+            # To handler derivative(expression), which is at least used in tests. Remove?
+            form_arguments = extract_arguments(form)
+
+        numbers = sorted(set(arg.number() for arg in form_arguments))
+        number = max(numbers + [-1]) + 1
+
+        # Don't know what to do with parts, let the user sort it out in that case
+        parts = set(arg.part() for arg in form_arguments)
+        ufl_assert(len(parts - {None}) == 0, "Not expecting parts here, provide your own arguments.")
+        part = None
+
+        # Create argument and split it if in a mixed space
         elements = [c.element() for c in coefficients]
         if len(elements) > 1:
             elm = MixedElement(*elements)
-            arguments = split(Argument(elm))
+            arguments = split(Argument(elm, number, part))
         else:
             elm, = elements
-            arguments = (Argument(elm),)
+            arguments = (Argument(elm, number, part),)
     else:
         # Wrap single argument in tuple for uniform treatment below
-        if isinstance(argument, (list,tuple)):
+        if isinstance(argument, (list, tuple)):
             arguments = tuple(argument)
         else:
             n = len(coefficients)
             if n == 1:
                 arguments = (argument,)
             else:
-                if argument.shape() == (n,):
+                if argument.ufl_shape == (n,):
                     arguments = tuple(argument[i] for i in range(n))
                 else:
                     arguments = split(argument)
 
     # Build mapping from coefficient to argument
     m = {}
-    for (c, a) in izip(coefficients, arguments):
-        ufl_assert(c.shape() == a.shape(), "Coefficient and argument shapes do not match!")
+    for (c, a) in zip(coefficients, arguments):
+        ufl_assert(c.ufl_shape == a.ufl_shape, "Coefficient and argument shapes do not match!")
         if isinstance(c, Coefficient):
             m[c] = a
         else:
             ufl_assert(isinstance(c, Indexed), "Invalid coefficient type for %s" % repr(c))
-            f, i = c.operands()
+            f, i = c.ufl_operands
             ufl_assert(isinstance(f, Coefficient), "Expecting an indexed coefficient, not %s" % repr(f))
             ufl_assert(isinstance(i, MultiIndex) and all(isinstance(j, FixedIndex) for j in i),
                        "Expecting one or more fixed indices, not %s" % repr(i))
@@ -190,32 +205,51 @@ def _handle_derivative_arguments(coefficient, argument):
             m[f][i] = a
 
     # Merge coefficient derivatives (arguments) based on indices
-    for c, p in m.iteritems():
+    for c, p in iteritems(m):
         if isinstance(p, dict):
-            a = zero_lists(c.shape())
-            for i, g in p.iteritems():
+            a = zero_lists(c.ufl_shape)
+            for i, g in iteritems(p):
                 set_list_item(a, i, g)
             m[c] = as_tensor(a)
 
     # Wrap and return generic tuples
     items = sorted(m.items(), key=lambda x: x[0].count())
-    coefficients = Tuple(*[item[0] for item in items])
-    arguments = Tuple(*[item[1] for item in items])
+    coefficients = ExprList(*[item[0] for item in items])
+    arguments = ExprList(*[item[1] for item in items])
     return coefficients, arguments
 
 def derivative(form, coefficient, argument=None, coefficient_derivatives=None):
     """UFL form operator:
-    Given any form, compute the linearization of the
-    form with respect to the given Coefficient.
+    Compute the Gateaux derivative of form w.r.t. coefficient in direction of argument.
+
+    If the argument is omitted, a new Argument is created
+    in the same space as the coefficient, with argument number
+    one higher than the highest one in the form.
+
     The resulting form has one additional Argument
     in the same finite element space as the coefficient.
+
     A tuple of Coefficients may be provided in place of
     a single Coefficient, in which case the new Argument
-    argument is based on a MixedElement created from this tuple."""
+    argument is based on a MixedElement created from this tuple.
 
-    coefficients, arguments = _handle_derivative_arguments(coefficient, argument)
+    An indexed Coefficient from a mixed space may be provided,
+    in which case the argument should be in the corresponding
+    subspace of the coefficient space.
 
-    coefficient_derivatives = coefficient_derivatives or {}
+    If provided, coefficient_derivatives should be a mapping from
+    Coefficient instances to their derivatives w.r.t. 'coefficient'.
+    """
+
+    coefficients, arguments = _handle_derivative_arguments(form, coefficient, argument)
+
+    if coefficient_derivatives is None:
+        coefficient_derivatives = ExprMapping()
+    else:
+        cd = []
+        for k in sorted_expr(coefficient_derivatives.keys()):
+            cd += [as_ufl(k), as_ufl(coefficient_derivatives[k])]
+        coefficient_derivatives = ExprMapping(*cd)
 
     # Got a form? Apply derivatives to the integrands in turn.
     if isinstance(form, Form):
